@@ -1,9 +1,7 @@
 /*
-$Id: shed.c,v 1.51 2009/03/24 13:47:26 alexsisson Exp $
+shed 1.16 source
 
-shed 1.15 source
-
-(C) Copyright 2002-2009 Alex Sisson (alexsisson@gmail.com)
+(C) Copyright 2002-2023 Alex Sisson (alexsisson@gmail.com)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -43,6 +41,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <sys/select.h> /* select     */
 #include <fcntl.h>      /* open       */
 #include <errno.h>
+#include <sys/time.h>   /* timeval    */
 
 #include "util.h"
 
@@ -64,12 +63,12 @@ void cursorright();
 void cursoradjust();
 int  cursorjump(off_t n);
 void clearline(int line, int width);
-int  search(char* str);
-int  getinput(char *prompt, int state, int acceptemptystr, int hexonly);
+int  search(const char* str);
+int  getinput(const char *prompt, int state, int acceptemptystr, int hexonly);
 int  mainloop();
 int  redraw();
 int  redraw_key_help(int onlykeyhelp);
-int  dump(char *dumpfile);
+int  dump(const char *dumpfile);
 
 /* globals */
 char   *filename;
@@ -91,6 +90,7 @@ int    coloffset[5]      = {0,6,10,14,18};                  /* offset from left
 char   *coltitle[5]      = {"asc","hex","dec","oct","bin"}; /* name of each column */
 char   *searchstr[5]     = {0,0,0,0,0};                     /* previous searches */
 char   *reply            = NULL;                            /* where input is returned */
+int    replymax          = 0;                               /* max length of reply */
 int    fdin              = 0;                               /* for dup()'ed stdin */
 fd_set fdset;                                               /* for select()ing fdin */
 char   *stdinbuf         = NULL;                            /* buf for stdin */
@@ -366,7 +366,7 @@ int main(int argc, char **argv) {
 
 int mainloop() {
 
-  int i,key;
+  int i, key, curx, cury;
   char s[256];
 
   while(1) {
@@ -407,8 +407,9 @@ int mainloop() {
           case 8:   /* ^H */
             if(strlen(reply)) {
               reply[strlen(reply)-1] = 0;
-              mvaddch(stdscr->_cury,stdscr->_curx-1,' ');
-              move(stdscr->_cury,stdscr->_curx-1);
+              getyx(stdscr, cury, curx);
+              mvaddch(cury, curx-1, ' ');
+              move(cury, curx-1);
             }
             break;
           default:
@@ -425,9 +426,14 @@ int mainloop() {
               key=0;
             }
             if(key) {
-              reply[strlen(reply)] = key;
-              addch(key);
-              refresh();
+              if(strlen(reply)+1 < replymax)
+              {
+                reply[strlen(reply)] = key;
+                addch(key);
+                refresh();
+              } else {
+                beep();
+              }
             }
             break;
           }
@@ -807,11 +813,10 @@ void clearline(int line, int width) {
 /* search */
 #define search_char_cmp(c1,c2) ((searchinsensitive && isalpha((c1))) ? tolower((c1))==(c2) || toupper((c1))==(c2) : (c1) == (c2))
 
-int search(char *str) {
+int search(const char *str) {
 
-  int c;
-  int i,slen;
-  unsigned char *search = NULL;
+  int c, i, search_len;
+  unsigned char *search_str = NULL;
   long l;
 
   if(!str)
@@ -826,25 +831,25 @@ int search(char *str) {
     return 0;
   }
 
-  slen = strlen(searchstr[cursorcol]);
-  search = malloc(slen+2);
-  strcpy((char*)search,searchstr[cursorcol]);
+  search_len = strlen(searchstr[cursorcol]);
+  search_str = malloc(search_len+2);
+  strcpy((char*)search_str, searchstr[cursorcol]);
 
   if(cursorcol) {
-    /* parse string */
-    char *p = malloc(slen+2);
-    strcpy(p,(char*)search);
+    /* non-ascii column - parse string */
+    char *p = malloc(search_len+2);
+    strcpy(p, (char*)search_str);
     p = strtok(p," :,.\0");
     for(i=0;p;i++) {
-      l = parsestring(p,colbase[cursorcol]);
+      l = parsestring(p, colbase[cursorcol]);
       if(l<0 || l>255)
         return 0;
-      search[i] = (unsigned char)l;
+      search_str[i] = (unsigned char)l;
       p = strtok(NULL," :,.\0");
     }
     free(p);
-    search[i] = 0;
-    slen = i;
+    search_str[i] = '\0';
+    search_len = i;
   }
 
   clearerr(f);
@@ -852,33 +857,33 @@ int search(char *str) {
   /* backwards */
   if(searchbackwards) {
 #ifdef LOG
-    fprintf(SHEDLOG,"search: backwards for '%s'\n",search);
+    fprintf(SHEDLOG,"search: backwards for '%s'\n", search);
 #endif
     cursorup(1);
-    SHED_FSEEK(f,cursorrow-1,SEEK_SET);
+    SHED_FSEEK(f, cursorrow-1, SEEK_SET);
     for(;cursorrow;cursorup(1)) {
       c = fgetc(f);
 #ifdef LOG
-      fprintf(SHEDLOG,"search: %d: c=%d(%c)\n",cursorrow,c,isprint(c)?c:'.');
+      fprintf(SHEDLOG,"search: %d: c=%d(%c)\n", cursorrow, c, isprint(c)?c:'.');
 #endif
-      if(search_char_cmp((unsigned char)c,search[0])) {
+      if(search_char_cmp((unsigned char)c, search_str[0])) {
 #ifdef LOG
         fprintf(SHEDLOG,"search:   first letter match:\n");
 #endif
-        for(i=1;i<slen;i++) {
+        for(i=1;i<search_len;i++) {
 #ifdef LOG
-          fprintf(SHEDLOG,"search:     c=%d(%c)\n",c,isprint(c)?c:'.');
+          fprintf(SHEDLOG,"search:     c=%d(%c)\n", c, isprint(c)?c:'.');
 #endif
           c = fgetc(f);
           if(c==EOF)
             break;
-          if(!search_char_cmp(c,search[i]))
+          if(search_char_cmp(c, search_str[i])==0)
             break;
         }
-        if(i==slen) {
+        if(i==search_len) {
           cursorup(i-1);
           cursordown(i-1);
-          free(search);
+          free(search_str);
           return 1;
         }
       }
@@ -892,18 +897,18 @@ int search(char *str) {
     SHED_FSEEK(f,cursorrow,SEEK_SET);
     for(;cursorrow<len-1;cursordown(1)) {
       c = fgetc(f);
-      if(search_char_cmp((unsigned char)c,search[0])) {
-        for(i=1;i<slen;i++) {
+      if(search_char_cmp((unsigned char)c, search_str[0])) {
+        for(i=1;i<search_len;i++) {
           c = fgetc(f);
           if(c==EOF)
             break;
-          if(!search_char_cmp(c,search[i]))
+          if(search_char_cmp(c,search_str[i])==0)
             break;
         }
-        if(i==slen) {
+        if(i==search_len) {
           cursordown(i-1);
           cursorup(i-1);
-          free(search);
+          free(search_str);
           return 1;
         }
         clearerr(f);
@@ -912,7 +917,7 @@ int search(char *str) {
     }
   }
 
-  free(search);
+  free(search_str);
   return 0;
 }
 
@@ -934,6 +939,13 @@ int cursorjump(off_t n) {
   return 0;
 }
 
+int calcwidth_status_cursor_row() {
+  /* Width of the "POSITION/LENGTH (base)" text on the right side of status line */
+  int base = decmode ? 10 : 16;
+  return calcwidth(cursorrow, base) + calcwidth(len, base) + 8;
+}
+
+
 int redraw() {
 
   int i,c,n;
@@ -1040,8 +1052,8 @@ int redraw() {
       mvaddstr(LINES-3,0,"(end)");
   }
 
-  c = decmode?10:16;
-  mvaddstr(LINES-3,COLS-(calcwidth(cursorrow,c)+calcwidth(len,c)+8),getstring(cursorrow,str,c,0));
+  c = decmode ? 10 : 16;
+  mvaddstr(LINES-3, COLS-calcwidth_status_cursor_row(), getstring(cursorrow,str,c,0));
   addstr("/");
   addstr(getstring(len,str,c,0));
   addstr((decmode)?" (dec)":" (hex)");
@@ -1099,9 +1111,9 @@ int redraw_key_help(int onlykeyhelp) {
 
 
 /* gets user input */
-int getinput(char *prompt, int state, int acceptemptystr, int hexonly) {
+int getinput(const char *prompt, int state, int acceptemptystr, int hexonly) {
   inputstate = state ? state : INPUTSTATE_GETSTR; //hexonly ? 2 : 1;
-  message = malloc(strlen(prompt)+5);
+  message = malloc(strlen(prompt)+3);
   if(!message)
     return -1;
   sprintf(message,"%s: ",prompt);
@@ -1110,10 +1122,11 @@ int getinput(char *prompt, int state, int acceptemptystr, int hexonly) {
   curs_set(1);
   attron(A_REVERSE);
   free(reply);
-  reply = malloc(128);
+  replymax = COLS - (strlen(message) + calcwidth_status_cursor_row());
+  reply = malloc(replymax);
   if(!reply)
     return -1;
-  memset(reply,0,128);
+  memset(reply,0,replymax);
   if(!mainloop())
     if(acceptemptystr||strlen(reply)) {
       attroff(A_REVERSE);
@@ -1122,12 +1135,13 @@ int getinput(char *prompt, int state, int acceptemptystr, int hexonly) {
   attroff(A_REVERSE);
   free(reply);
   reply = NULL;
+  replymax = 0;
   return -1;
 }
 
 
 /* dump shed output to another file */
-int dump(char *dumpfile) {
+int dump(const char *dumpfile) {
 
   int i,c;
   FILE *df;
