@@ -1,7 +1,5 @@
 /*
-$Id: util.c,v 1.9 2009/03/23 12:42:15 alexsisson Exp $
-
-(C) Copyright 2002-2004 Alex Sisson (alexsisson@gmail.com)
+(C) Copyright 2002-2023 Alex Sisson (alexsisson@gmail.com)
 
 This file is part of shed.
 
@@ -30,11 +28,17 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "util.h"
 
 /* ascii control char descs */
-char  ascii_short_desc[] = {'0',0,0,0,0,0,0,'a','b','t','n','v','f','r'};
-char *ascii_long_desc[]  = {"NUL","SOH","STX","ETX","EOT","ENQ","ACK","BEL",
-                            "BS ","HT ","LF ","VT ","FF ","CR ","SO ","SI ",
-                            "DLE","DC1","DC2","DC3","DC4","NAK","SYN","ETB",
-                            "CAN","EM ","SUB","ESC","FS ","GS ","RS ","US ","SPC"};
+char  ascii_short_desc[] = {  '0',     0,     0,     0,     0,     0,     0,   'a',
+                              'b',   't',   'n',   'v',   'f',   'r',     0,     0,
+                                0,     0,     0,     0,     0,     0,     0,     0,
+                                0,     0,     0,     0,     0,     0,     0,     0,
+                                0};
+
+char *ascii_long_desc[]  = {"NUL", "SOH", "STX", "ETX", "EOT", "ENQ", "ACK", "BEL",
+                            "BS ", "HT ", "LF ", "VT ", "FF ", "CR ", "SO ", "SI ",
+                            "DLE", "DC1", "DC2", "DC3", "DC4", "NAK", "SYN", "ETB",
+                            "CAN", "EM ", "SUB", "ESC", "FS ", "GS ", "RS ", "US ",
+                            "SPC"};
 
 
 
@@ -42,11 +46,12 @@ char *ascii_long_desc[]  = {"NUL","SOH","STX","ETX","EOT","ENQ","ACK","BEL",
 char *getascii(unsigned char c, char *s, int mode)
 {
   if(c>127)
-    strncpy(s,"   ",3);
+    strcpy(s, "   ");
   else if(c>=32 && c<=126) {
     s[0] = ' ';
     s[1] = c;
     s[2] = ' ';
+    s[3] = '\0';
   } else {
     switch(mode) {
       case 1: /* do c-style control chars */
@@ -54,19 +59,19 @@ char *getascii(unsigned char c, char *s, int mode)
           s[0] = ' ';
           s[1] = '\\';
           s[2] = ascii_short_desc[c];
+          s[3] = '\0';
           break;
         }
 
       case 0: /* no special chars */
-        strncpy(s,"   ",3);
+        strcpy(s, "   ");
         break;
 
       case 2: /* 3 letter descs */
-        strncpy(s,c!=127?ascii_long_desc[c]:"DEL",3);
+        strcpy(s, c!=127 ? ascii_long_desc[c] : "DEL");
         break;
     }
   }
-  s[3] = 0;
   return s;
 }
 
