@@ -1,7 +1,7 @@
 /*
-$Id: util.h,v 1.4 2004/10/16 15:16:38 alexsisson Exp $
+$Id: util.h,v 1.6 2009/01/12 02:45:26 alexsisson Exp $
 
-(C) Copyright 2002-2003 Alex Sisson (alexs@cyberspace.org)
+(C) Copyright 2002-2003 Alex Sisson (alexsisson@gmail.com)
 
 This file is part of shed.
 
@@ -24,6 +24,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define UTIL_H_INCLUDED
 
 #include <getopt.h>
+#include <stdint.h>
 
 /* defines */
 #define strequ(s1,s2) (!(strcmp(s1,s2)))
@@ -34,9 +35,9 @@ extern char *ascii_long_desc[];
 
 /* functions */
 char *getascii(unsigned char c, char *s, int mode);
-char *getstring(unsigned int n, char *s, int base, int width);
-int parsestring(char *s, int base);
-int calcwidth(unsigned int n, int base);
+char *getstring(uint64_t n, char *s, int base, int width);
+int64_t parsestring(char *s, int base);
+int calcwidth(uint64_t n, int base);
 char *getopt_makeoptstring(struct option *opt);
 
 #endif
